SystemOrganization addCategory: #DijEvolution!
SystemOrganization addCategory: #'DijEvolution-Test'!

!Integer methodsFor: '*DijEvolution' stamp: 'hjo 6/27/2012 15:03'!
multiply: aSmallInteger modulo: aSmallInteger2 

	^(self * aSmallInteger) \\ aSmallInteger2! !

Object subclass: #DijEvolution
	instanceVariableNames: 'ancestors leastCostLife breedingCode childrenOfLeastCostLife costSortedLives ancestorObjects dicFromObjectToCostSortedLife'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DijEvolution'!

!DijEvolution classSide methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 11:11'!
forGraph

	^self new
		 breedingCode: 
			[:parent| |v nei|
			v:= parent object.
			nei:= v neighbors.
			nei collect: [:n|  |e|
				e:= (v edgesConnectingTo: n) anyOne.  
				parent giveBirthWith: (Message selector:#otherEndPointVia: argument: e) cost: e cost  ].
			].! !

!DijEvolution classSide methodsFor: 'examples' stamp: 'hjo 6/28/2012 11:14'!
modulo17
|evo sol|
evo := DijEvolution new.
evo addSeed: 1.
evo breedingCode: [:parent| #(2 3) collect: [:m| parent giveBirthWith:(Message selector:#multiply:modulo: arguments: (Array with: m with: 17)) ] ].  
evo step.
evo runUntilEqual: 7.

sol := evo ancestors last.

sol birthMessageChain.
sol birthMessagesReplayed.
sol object.
sol cost.  ! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 7/22/2012 03:21'!
addChildrenToCostSortedLives

	childrenOfLeastCostLife do: [:child|
		| sameObjectLife |
		sameObjectLife := dicFromObjectToCostSortedLife at: (child object) ifAbsent: [#notFound].
		sameObjectLife = #notFound
			ifTrue: [self addLifeToCostSortedLives: child]
			ifFalse: 
				[sameObjectLife cost > child cost
					ifTrue: 
						[self removeFromCostSortedLives: sameObjectLife.
						self addLifeToCostSortedLives: child]].
		]! !

!DijEvolution methodsFor: 'adding' stamp: 'hjo 7/22/2012 03:12'!
addLifeToCostSortedLives: aDijLife

	costSortedLives add: aDijLife.
	dicFromObjectToCostSortedLife at: (aDijLife object) put: aDijLife.! !

!DijEvolution methodsFor: 'adding' stamp: 'hjo 7/22/2012 03:29'!
addSeed: anObject

	| seedLife |
	seedLife := (DijLife object: anObject cost: 0).
	costSortedLives add: seedLife.
	dicFromObjectToCostSortedLife at: (seedLife object) put: seedLife.! !

!DijEvolution methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:49'!
ancestors

	^ ancestors! !

!DijEvolution methodsFor: 'accessing' stamp: 'hjo 6/28/2012 00:38'!
breedingCode: aBlock
	"[:parent| #(2 3) collect: [:m| parent giveBirthWith:(Message selector:#multiply:modulo: arguments: (Array with: m with: 17)) ] ]"
	breedingCode := aBlock
! !

!DijEvolution methodsFor: 'testing' stamp: 'hjo 6/28/2012 11:59'!
canStep

	^costSortedLives notEmpty! !

!DijEvolution methodsFor: 'accessing' stamp: 'hjo 6/28/2012 11:59'!
costSortedLives

	^ costSortedLives! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 7/8/2012 14:06'!
dropChildren
	childrenOfLeastCostLife := childrenOfLeastCostLife reject: [ :ch | ancestorObjects includes: ch object ]! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 7/10/2012 17:11'!
getLeastCostLifeFromSorted
	leastCostLife := self removeFromCostSortedLives: costSortedLives first.! !

!DijEvolution methodsFor: 'initialize-release' stamp: 'hjo 7/22/2012 03:11'!
initialize

	super initialize.
	ancestors := Set new.
	ancestorObjects := Set new.
	costSortedLives := SortedCollection sortBlock: (UnarySortDescriptor ascendingBy: #cost).
	dicFromObjectToCostSortedLife := Dictionary new.! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 7/8/2012 13:52'!
jionLeastCostLifeToAncestors
	ancestors add: leastCostLife.
	ancestorObjects add: leastCostLife object.! !

!DijEvolution methodsFor: 'accessing' stamp: 'hjo 7/8/2012 18:28'!
leastCostLife

	^ leastCostLife! !

!DijEvolution methodsFor: 'subroutines' stamp: 'hjo 6/28/2012 01:21'!
makeChildren
	childrenOfLeastCostLife := breedingCode value: leastCostLife.! !

!DijEvolution methodsFor: 'adding' stamp: 'HwaJong 11/5/2012 22:49'!
removeFromCostSortedLives: aDijLife

	| removed |
	removed := costSortedLives remove: aDijLife.
	self flag: 'It turns out, that aDictionary >> removeKey: is the bottle-neck.'.
	dicFromObjectToCostSortedLife removeKey: (aDijLife object).
	^removed! !

!DijEvolution methodsFor: 'stepping and presenter' stamp: 'hjo 6/28/2012 09:01'!
run

	[self canStep ifTrue: [self step. true] ifFalse: [false]
	] whileTrue.! !

!DijEvolution methodsFor: 'stepping and presenter' stamp: 'hjo 7/8/2012 13:52'!
runUntil: aBlock

	| result |
	(ancestorObjects anySatisfy: [:each| aBlock value: each]) ifTrue: [^true].
	[
		self canStep and: [self step. result := aBlock value: leastCostLife object. result not ]
	] whileTrue.
	^result.! !

!DijEvolution methodsFor: 'stepping and presenter' stamp: 'hjo 7/4/2012 11:37'!
runUntilEqual: anObject

	^self runUntil: [:l | l = anObject]! !

!DijEvolution methodsFor: 'stepping and presenter' stamp: 'hjo 7/5/2012 07:39'!
step

	self getLeastCostLifeFromSorted.
	self jionLeastCostLifeToAncestors.
	self makeChildren.
	self dropChildren.
	self addChildrenToCostSortedLives.
! !

Object subclass: #DijLife
	instanceVariableNames: 'origin birthMessage birthCost cost object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DijEvolution'!

!DijLife classSide methodsFor: 'instance creation' stamp: 'hjo 6/28/2012 16:13'!
object: anObject 

	^self object: anObject cost: 0! !

!DijLife classSide methodsFor: 'instance creation' stamp: 'hjo 6/27/2012 14:18'!
object: anObject cost: aNumber

	^self new object: anObject; cost: aNumber! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:53'!
birthCost
	^ birthCost! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/28/2012 00:53'!
birthCost: aNumber
	birthCost := aNumber! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:53'!
birthMessage
	^ birthMessage! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/28/2012 00:53'!
birthMessage: aMessage
	birthMessage := aMessage! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 21:58'!
birthMessageChain

	^Array streamContents: [:stream| self birthMessageChainOn: stream]! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 21:58'!
birthMessageChainOn: stream

	origin notNil
		ifTrue: [ origin birthMessageChainOn: stream. stream nextPut: birthMessage ].! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 21:52'!
birthMessagesReplayed

	origin isNil ifTrue: [^object].
	^birthMessage sendTo: origin birthMessagesReplayed. ! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 14:59'!
computeCost

	cost := origin cost + birthCost.! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 04:08'!
cost
	^ cost! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 04:08'!
cost: aNumber
	cost := aNumber
! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 23:14'!
giveBirthWith: aMessage 

	^self giveBirthWith: aMessage cost: 1! !

!DijLife methodsFor: 'as yet unclassified' stamp: 'hjo 6/27/2012 23:13'!
giveBirthWith: aMessage cost: aNumber

	^self species new 
		origin: self;
		birthMessage: aMessage;
		birthCost: aNumber;
		object: (aMessage sendTo: object);
		computeCost  ! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 14:50'!
object

	^ object! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:57'!
object: anObject 

	object := anObject! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:53'!
origin
	^ origin! !

!DijLife methodsFor: 'accessing' stamp: 'hjo 6/27/2012 13:53'!
origin: anObject
	origin := anObject! !

!DijLife methodsFor: 'printing' stamp: 'hjo 6/27/2012 18:03'!
printOn: aStream

	aStream nextPut: $[.
	aStream nextPutAll: 'o:'.
	aStream print: object.
	aStream space.
	aStream nextPutAll: 'c:'.
	aStream print: cost.
	aStream nextPut: $].! !

TestCase subclass: #DijEvolutionTest
	instanceVariableNames: 'evo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DijEvolution-Test'!

!DijEvolutionTest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 14:43'!
setUp

	super setUp.
	evo := DijEvolution new.
	evo addSeed: 1.
	evo breedingCode: [:parent| #(2 3) collect: [:m| parent giveBirthWith:(Message selector:#multiply:modulo: arguments: (Array with: m with: 17)) ] ].  ! !

!DijEvolutionTest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 14:48'!
testRun

	evo run.
	self assert: 16 equals: evo ancestors size.
	self assert: 0 equals: evo costSortedLives size.! !

!DijEvolutionTest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 14:46'!
testRunUntilEqual

	evo runUntilEqual: 7.
	self assert: 11 equals: evo ancestors size.
	self assert: 3 equals: evo costSortedLives size.! !

!DijEvolutionTest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 14:44'!
testSetUp

	self assert: 0 equals: evo ancestors size.
	self assert: 1 equals: evo costSortedLives size.! !

!DijEvolutionTest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 14:45'!
testStep

	evo step.
	self assert: 1 equals: evo ancestors size.
	self assert: 2 equals: evo costSortedLives size.! !

TestCase subclass: #DijLifetest
	instanceVariableNames: 'life'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DijEvolution-Test'!

!DijLifetest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 16:10'!
setUp

	super setUp.
	life := DijLife object: 'hello'.! !

!DijLifetest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 23:20'!
testBirthMessageChain

	| rev short upp m1 m2 m3 |
	rev := life giveBirthWith: (m1 := Message selector: #reverse) cost: 3.
	short := rev giveBirthWith: (m2 := Message selector: #copyFrom:to: arguments: (Array with: 2 with: 4)) cost: 7.
	upp := short giveBirthWith: (m3 := Message selector: #asUppercase) cost: 2.
	self assert: (Array with: m1 with: m2 with: m3) equals: upp birthMessageChain.! !

!DijLifetest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 16:14'!
testGiveBirthWithCost

	| rev |
	rev := life giveBirthWith: (Message selector: #reverse) cost: 3.
	self assert: 'olleh' equals: rev object.
	self assert: 3 equals: rev cost.! !

!DijLifetest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 16:16'!
testGiveBirthWithCost2

	| rev short |
	rev := life giveBirthWith: (Message selector: #reverse) cost: 3.
	short := rev giveBirthWith: (Message selector: #copyFrom:to: arguments: (Array with: 2 with: 4)) cost: 7.
	self assert: 'lle' equals: short object.
	self assert: 10 equals: short cost.! !

!DijLifetest methodsFor: 'as yet unclassified' stamp: 'hjo 6/28/2012 16:14'!
testSetUp

	self assert: 'hello' equals: life object.
	self assert: 0 equals: life cost.! !
